<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Controller;

use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationAwareInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationInterface;
use Sylius\Bundle\ResourceBundle\Controller\AuthorizationCheckerInterface;
use Sylius\Bundle\ResourceBundle\Controller\RequestConfiguration;
use Sylius\Component\Core\Model\AdminUserInterface;
use Sylius\Component\Customer\Context\CustomerContextInterface;
use Symfony\Component\Security\Core\Security;

class QuoteAuthorizationChecker implements AuthorizationCheckerInterface, QuoteConfigurationAwareInterface
{
    private readonly QuoteConfigurationInterface $quoteConfiguration;

    private readonly Security $security;

    private readonly AuthorizationCheckerInterface $decoratedAuthorizationChecker;

    private readonly CustomerContextInterface $customerContext;

    public function __construct(
        QuoteConfigurationInterface $quoteConfiguration,
        Security $security,
        AuthorizationCheckerInterface $decoratedAuthorizationChecker,
        CustomerContextInterface $customerContext,
    ) {
        $this->quoteConfiguration = $quoteConfiguration;
        $this->security = $security;
        $this->decoratedAuthorizationChecker = $decoratedAuthorizationChecker;
        $this->customerContext = $customerContext;
    }

    public function isGranted(RequestConfiguration $configuration, string $permission): bool
    {
        if ($this->getQuoteConfiguration()->isAnonymousQuoteAllowed()) {
            return true;
        }

        if ($this->security->getUser() instanceof AdminUserInterface) {
            return true;
        }

        if ('dedi_quote.quote.update' === $permission) {
            return $this->customerIsGranted();
        }

        return $this->decoratedAuthorizationChecker->isGranted($configuration, $permission);
    }

    private function customerIsGranted(): bool
    {
        return null !== $this->customerContext->getCustomer();
    }

    public function getQuoteConfiguration(): QuoteConfigurationInterface
    {
        return $this->quoteConfiguration;
    }
}
